.aeabi_subsection aeabi_pauthabi, required, uleb128
.aeabi_attribute Tag_PAuth_Schema, 1

.aeabi_subsection aeabi_feature_and_bits, optional, uleb128
.aeabi_attribute Tag_Feature_BTI, 1
.aeabi_attribute Tag_Feature_PAC, 0
.aeabi_attribute Tag_Feature_GCS, 1

.aeabi_subsection foo_1, optional, uleb128
.aeabi_attribute 1, 1

.aeabi_subsection foo_2, optional, uleb128
.aeabi_attribute 0, 1
.aeabi_attribute 1, 1

.aeabi_subsection foo_1, optional, uleb128
.aeabi_attribute 2, 0

.aeabi_subsection foo_2, optional, uleb128
.aeabi_attribute 1, 1

.aeabi_subsection foo_1, optional, uleb128
.aeabi_attribute 2, 0

.aeabi_subsection foo_1, optional, uleb128
.aeabi_attribute 3, 0
.aeabi_attribute 4, 0

.aeabi_subsection foo_3, optional, ntbs
.aeabi_attribute 0, "hello"
.aeabi_attribute 1, "world"
