// parse-only

1 MENU
{
    POPUP "&File"
    {
        MENUITEM "This is a test #1", 100, BITMAP
        MENUITEM "This is a test #2", 101, OWNERDRAW
        MENUITEM "This is a test #3", 102, CHECKED
        MENUITEM "This is a test #4", 103, GRAYED
        MENUITEM "This is a test #5", 104, HELP
        MENUITEM "This is a test #6", 105, INACTIVE
        MENUITEM "This is a test #7", 106, MENUBARBREAK
        MENUITEM "This is a test #8", 107, MENUBREAK
    }
}
